<?php

require 'SsoConf.php';
require 'SsoCode.php';
require 'SsoSocket.php';

class Sso {
	public $hashmap = array();
	
	public function getUserId() {
		if(array_key_exists(SsoCode::$KEY_USER_ID, $this->hashmap) == false)
			return null;
		else 
			return $this->hashmap[SsoCode::$KEY_USER_ID];
	}
	
	public function setUserId($userId) {
		$this->hashmap[SsoCode::$KEY_USER_ID] = $userId;
	}
	
	public function getClientIp() {
		if(array_key_exists(SsoCode::$KEY_CLIENT_IP, $this->hashmap) == false)
			return null;
		else 
			return $this->hashmap[SsoCode::$KEY_CLIENT_IP];
	}
	
	public function setClientIp($clientIp) {
		$this->hashmap[SsoCode::$KEY_CLIENT_IP] = $clientIp;
	}
	
	public function getProfile($key) {
		if(array_key_exists($key, $this->hashmap) == false)
			return null;
		else 
			return $this->hashmap[$key];
	}
	
	public function setProfile($key, $value) {
		$this->hashmap[$key] = $value;
	}
	
	public function getToken() {
		if(array_key_exists(SsoCode::$KEY_TOKEN, $this->hashmap) == false)
			return null;
		else 
			return $this->hashmap[SsoCode::$KEY_TOKEN];
	}
	
	public function setToken($token) {
		$this->hashmap[SsoCode::$KEY_TOKEN] = $token;
	}

	public function getAgentIp() {
		return SsoConf::$AGENT_IP;
	}
	
	public function getAgentPort() {
		return SsoConf::$AGENT_PORT;
	}
	
	public function setAgentPort($port) {
		$this->hashmap[SsoConf::$AGENT_PORT] = $port;
	}
	
	public function getServerIp() {
		return SsoConf::$SERVER_IP;
	}
	
	public function setServerIp($ip) {
		$this->hashmap[SsoConf::$SERVER_IP] = $ip;
	}
		
	public function getServerPort() {
		return SsoConf::$SERVER_PORT;
	}
	
	public function setServerPort($port) {
		$this->hashmap[SsoConf::$SERVER_PORT] = $port;
	}
	
	public function getError() {
		if(array_key_exists(SsoCode::$KEY_ERROR, $this->hashmap) == false)
			return null;
		else 
			return $this->hashmap[SsoCode::$KEY_ERROR];
	}
	
	public function setError($error) {
		$this->hashmap[SsoCode::$KEY_ERROR] = $error;
	}
	
	public function getWarn() {
		if(array_key_exists(SsoCode::$KEY_WARN, $this->hashmap) == false)
			return null;
		else 
			return $this->hashmap[SsoCode::$KEY_WARN];
	}
	
	public function setWarn($warn) {
		$this->hashmap[SsoCode::$KEY_WARN] = $warn;
	}
	
	// 토큰 생성
	public function issueToken() {
		// 요청 유효성 체크
		do {
			// 토큰 생성 요청에 아이디가 없는 경우(필수값)
			if(empty($this->getUserId())) {
				$this->hashmap = array();
				$this->setError(SsoCode::$ERR_USERID_NOT_SET);
				return;
			}
			// 토큰 생성 요청에 CLIENT_IP가 없는 경우(필수값)
			if(empty($this->getClientIp())) {
				$this->hashmap = array();
				$this->setError(SsoCode::$ERR_CLIENT_IP_NOT_SET);
				return;
			}
		} while(false);
		
		// 토큰 생성시간
		// 시간 안 맞으면 php.ini의 date.timezone 설정
		$nowDate = date('Y-m-d H:i:s');
		$this->hashmap[SsoCode::$KEY_CREATE_TIME] = $nowDate;
		
		// DATA1 생성
		$data1Hashmap = array();
		$data1Hashmap[SsoCode::$KEY_CLIENT_IP] = $this->getClientIp();
		$data1 = json_encode($data1Hashmap);
		$dataLength1 = strlen($data1);

		// DATA2 생성
		$data2 = json_encode($this->hashmap);
		$dataLength2 = strlen($data2);

		// SSO Server와 소켓통신
		$socket;
		try {
			$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
			$result = socket_connect($socket, SsoConf::$SERVER_IP, SsoConf::$SERVER_PORT);
			
			// 1. Server --> API 인사말 수신
			$ssoSocket = new SsoSocket();
			$ssoSocket->recvHello($socket);

			// 2. API --> Server 요청 전송
			$temp = new SsoProtocol();
			$temp->setValue(SsoCode::$SSO_API, SsoCode::$SSO_SERVER, SsoCode::$CMD_ISSUE_TOKEN, 0x00, 0x00, $dataLength1, $dataLength2, $data1, $data2);
			$ssoSocket->send($socket, $temp);
			
			// 3. Server --> API 요청 수신
			$ssoProtocol = $ssoSocket->recv($socket);
	
			// 결과 SSO 객체 구성
			$this->hashmap = array();
			$this->hashmap = array_merge(json_decode($ssoProtocol->data2, true), $this->hashmap);
			$this->setError($ssoProtocol->error);
			$this->setWarn($ssoProtocol->warn);
			
		} catch (Exception $e) {
			$this->setError(SsoCode::$ERR_SERVER_CONNECTION_FAIL);
		} finally {
			if(empty($socket) == false) socket_close($socket);
		}
	}
	
	// 토큰검증(복호화)
	public function verifyToken() {
		// 요청 유효성 체크
		do {
			// 토큰 생성 요청에 CLIENT_IP가 없는 경우(필수값)
			if(empty($this->getClientIp())) {
				$this->hashmap = array();
				$this->setError(SsoCode::$ERR_CLIENT_IP_NOT_SET);
				return;
			}
			// 토큰 검증 요청에 TOKEN이 없는 경우(필수값)
			if(empty($this->getToken())) {
				$this->hashmap = array();
				$this->setError(SsoCode::$ERR_TOKEN_NOT_SET);
				return;
			}
		} while(false);
			
		// DATA1 생성
		$data1Hashmap = array();
		$data1Hashmap[SsoCode::$KEY_CLIENT_IP] = $this->getClientIp();
		$data1 = json_encode($data1Hashmap);
		$dataLength1 = strlen($data1);
		
		// DATA2 생성
		$data2 = json_encode($this->hashmap);
		$dataLength2 = strlen($data2);
		
		// SSO Server와 소켓통신
		$socket;
		try {
			$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
			$result = socket_connect($socket, SsoConf::$AGENT_IP, SsoConf::$AGENT_PORT);
			
			// 1. Agent --> API 인사말 수신
			$ssoSocket = new SsoSocket();
			$ssoSocket->recvHello($socket);

			// 2. API --> Agent 요청 전송
			$temp = new SsoProtocol();
			$temp->setValue(SsoCode::$SSO_API, SsoCode::$SSO_SERVER, SsoCode::$CMD_VERIFY_TOKEN, 0x00, 0x00, $dataLength1, $dataLength2, $data1, $data2);
			$ssoSocket->send($socket, $temp);
			
			// 3. Agent --> API 요청 수신
			$ssoProtocol = $ssoSocket->recv($socket);
	
				
			// 응답 유효성 체크
			do {
				// 서버에서 응답을 안보내줄 경우 에러처리
				if(empty($ssoProtocol->data1) || empty($ssoProtocol->data2)) {
					$this->hashmap = array();
					$this->setError(SsoCode::$ERR_NO_RESPONSE);					
					return;
				}
			}
			while(false);
			
			// 결과 SSO 객체 구성
			$this->hashmap = array();
			$this->hashmap = array_merge(json_decode($ssoProtocol->data2, true), $this->hashmap);
			$this->setError($ssoProtocol->error);
			$this->setWarn($ssoProtocol->warn);
			
		} catch (Exception $e) {
			$this->setError(SsoCode::$ERR_AGENT_CONNECTION_FAIL);
		} finally {
			if(empty($socket) == false) socket_close($socket);
		}
	}
}
?>